/***************************************************************************
 *            balle.h
 *
 *  Thu Nov 22 09:44:58 2007
 *  Copyright  2007  Daniel CALECA
 *  daniel.caleca@etumel.univmed.fr
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef H_CB_BALL
#define H_CB_BALL

#include "item.h"
#include "joueur.h"

class Balle:public Item
{
  private:
    float rayon;
  Joueur *proprietaire;
  Vecteur vitesse;
  bool traversante;
  bool accrochee;
  
  int indiceRayon;
  GLUquadric *quadric;
  
  static const int indiceRmax = 3;
  static const int rayonInit = 1;
  static const float pasRayon = 0.2;
  static const float vitesseInit = 0.6;
  
  public:
    
    Balle (Joueur *prop,float X, float Y,bool hooked);
  ~Balle ();
  
  inline Joueur *getProp ()
  {return proprietaire;}
  
  void afficher ();

  inline void deplacer ()
  {
    position = vitesse + position;
  }
  inline void deplacer (Vecteur distance)
  {
    position = distance + position;
  }
    
  inline void undoDeplacer()
  {
    position = position - vitesse;
  }
  
  inline bool isHooked()
  {
    return accrochee;
  }
  
  inline void lancer()
  {
    accrochee = false;
  }
  
  inline void suivreBatte(int sens)
  {
    if(sens >0)
      position.X = position.X + proprietaire->getBatteRef().getPasDep();
    else
      position.X = position.X - proprietaire->getBatteRef().getPasDep();
  }
  
  void agrandir ();
  void reduire ();
  
  float getRayon ();
  int getIRayon ();
  
  inline bool isGoingDown()
  {
    if(vitesse.Y <= 0)
      return true;
    else
      return false;
  }

  inline Vecteur* getVitesse()
  {
    return &vitesse;
  }

  inline void rebondBatte(float angle)
  {
    vitesse.X = vitesse.norme * sinf(angle);
    vitesse.Y = vitesse.norme * cosf(angle);
  }
  
  inline void stop()
  {
	vitesse.X = 0;
	vitesse.Y = 0;
  }
  
  inline void setTraversante()
  {
    traversante=true;
  }
  inline bool isTraversante()
  {
    return traversante;
  }
  
  inline void avanceX()
  {position.X += vitesse.X;}
  inline void avanceY()	
  {position.Y += vitesse.Y;}
  
  inline void reculeX()
  {position.X -= vitesse.X;}
  inline void reculeY()	
  {position.Y -= vitesse.Y;}
  
  inline void rebondX()
  {vitesse.X = - vitesse.X;}
  inline void rebondY()	
  {vitesse.Y = - vitesse.Y;}
  
  inline float getVitesseInit()
  {return vitesseInit;}
  
  void collision (Vecteur normale);
  
};

#endif
