/***************************************************************************
 *            item.h
 *
 *  Thu Nov 22 09:42:55 2007
 *  Copyright  2007  Daniel CALECA
 *  daniel.caleca@etumel.univmed.fr
 ****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 


#ifndef H_CB_ITEMS
#define H_CB_ITEMS

#include <math.h>
#include <iostream>

typedef enum SIDE {
  COTE_HAUT=1,COTE_BAS
}Side;

class BoiteCollision
{
   public:
   float lX;
   float lY;
   float lZ;
   
};

class Vecteur
{
  public:
    float X;
  float Y;
  float Z;
  float norme;
  
  Vecteur()
  {
    X=0;
    Y=0;
    Z=0;
    norme=-1;
  }

  //normalise le vecteur
  inline void normalize() //3e dim ignoree
  {
    float N = sqrt( powf(X,2) + powf(Y,2));
    norme = 1;
    X = X / N;
    Y = Y / N;
//    Z = Z / N;
  }

  inline void setNorme(float nNorme)//3e dim ignoree
  {
    if(norme == -1)
      std::cout << "(WW) norme du vecteur non initialisee !! (fonction setNorme())" <<std::endl;
    float rapport = nNorme / norme;
    norme = nNorme;
    
    X = X * rapport;
    Y = Y * rapport;
//    Z = Z * rapport;
  }
  
  Vecteur operator+(const Vecteur &v){
    Vecteur res;
    res.X = X + v.X;
    res.Y = Y + v.Y;
    res.Z = Z + v.Z;
    return res;
  }
  
  
  Vecteur operator-(const Vecteur &v){
    Vecteur res;
    res.X = X - v.X;
    res.Y = Y - v.Y;
    res.Z = Z - v.Z;
    return res;
  }
  
    Vecteur operator-(){
    Vecteur res;
    res.X = -X ;
    res.Y = -Y ;
    res.Z = -Z ;
    return res;
  }
  
  //calcul des rebonds sur surface plane classique
  Vecteur operator*(const Vecteur &v){ //version 2d, 3D inutile
    Vecteur res;
    
    float angle;
    float normedV;

    res.norme = norme;
    //optimisations locales pour les collisions selons Ox ou Oy
    if((v.X == 1 || v.X == -1) && v.Y == 0)
    {
      res.X=-X;
      res.Y=Y;
    }
    else if((v.Y == 1 || v.Y == -1) && v.X == 0)
    {
      res.Y=-Y;
      res.X=X;
    }
    else
    {
      angle = atan2(v.Y,v.X);
      normedV = -2* (X*v.X + Y*v.Y);

      if(fabs(angle) <= 0.01)
        angle = 0;

      res.X= X + normedV * cosf(angle);
      res.Y= Y + normedV * sinf(angle);
    //res.Z = 0;
    }

    return res;
    
  }
  
  Vecteur & operator=(const Vecteur &v){
    X = v.X;
    Y = v.Y;
    Z = v.Z;
    return *this;
  }
};


class Item
{
 protected:

   Item(float X,float Y,float aX,float aY,float aZ)
    {
        position.X=X;
        position.Y=Y;
        position.Z=0;
        position.norme=-1;
        angleX=aX;
        angleY=aY;
        angleZ=aZ;
    }
    Item(Vecteur pos,float aX,float aY,float aZ)
    {
        position=pos;
        angleX=aX;
        angleY=aY;
        angleZ=aZ;
    }
    virtual ~Item() {}
   Vecteur position;

   float angleX;
   float angleY;
   float angleZ;

 public:
   inline Vecteur* getPosition()
      {return &position;}
   virtual void afficher() = 0;
};

#endif
