/***************************************************************************
 *            menu.cc
 *
 *  Tue Dec  4 10:43:58 2007
 *  Copyright  2007  MUZI Jean-Marc
 *  jean-marc.muzi@etumel.univmed.fr
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "FreeType.h"
#include "menu.h"
#include "typesparties.h"

#include <iostream>

#include "partie.h"

#define LARGEUR_ECRAN 800
#define HAUTEUR_ECRAN 600


using namespace std;

/**************************************************************************/
// fonctions de chargement de texture
/*************************************************************************/
SDL_Surface * flipSurface(SDL_Surface * surface)
{
    int current_line,pitch;
    SDL_Surface * fliped_surface = SDL_CreateRGBSurface(SDL_SWSURFACE,
                                   surface->w,surface->h,
                                   surface->format->BitsPerPixel,
                                   surface->format->Rmask,
                                   surface->format->Gmask,
                                   surface->format->Bmask,
                                   surface->format->Amask);



    SDL_LockSurface(surface);
    SDL_LockSurface(fliped_surface);

    pitch = surface->pitch;
    for (current_line = 0; current_line < surface->h; current_line ++)
    {
        memcpy(&((unsigned char* )fliped_surface->pixels)[current_line*pitch],
               &((unsigned char* )surface->pixels)[(surface->h - 1  -
                                                    current_line)*pitch],
               pitch);
    }

    SDL_UnlockSurface(fliped_surface);
    SDL_UnlockSurface(surface);
    return fliped_surface;
}

static GLuint loadTexture(const char * filename,bool useMipMap)
{
    GLuint glID;
    SDL_Surface * picture_surface = NULL;
    SDL_Surface *gl_surface = NULL;
    SDL_Surface * gl_fliped_surface = NULL;
    Uint32 rmask, gmask, bmask, amask;

    picture_surface = IMG_Load(filename);
    if (picture_surface == NULL)
        return 0;

#if SDL_BYTEORDER == SDL_BIG_ENDIAN

    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;
#else

    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
    amask = 0xff000000;
#endif

    SDL_PixelFormat format = *(picture_surface->format);
    format.BitsPerPixel = 32;
    format.BytesPerPixel = 4;
    format.Rmask = rmask;
    format.Gmask = gmask;
    format.Bmask = bmask;
    format.Amask = amask;

    gl_surface = SDL_ConvertSurface(picture_surface,&format,SDL_SWSURFACE);

    gl_fliped_surface = flipSurface(gl_surface);

    glGenTextures(1, &glID);

    glBindTexture(GL_TEXTURE_2D, glID);


    if (useMipMap)
    {

        gluBuild2DMipmaps(GL_TEXTURE_2D, 4, gl_fliped_surface->w,
                          gl_fliped_surface->h, GL_RGBA,GL_UNSIGNED_BYTE,
                          gl_fliped_surface->pixels);

        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,
                        GL_LINEAR_MIPMAP_LINEAR);

    }
    else
    {
        glTexImage2D(GL_TEXTURE_2D, 0, 4, gl_fliped_surface->w,
                     gl_fliped_surface->h, 0, GL_RGBA,GL_UNSIGNED_BYTE,
                     gl_fliped_surface->pixels);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    }
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);


    SDL_FreeSurface(gl_fliped_surface);
    SDL_FreeSurface(gl_surface);
    SDL_FreeSurface(picture_surface);

    return glID;
}

/**************************************************************************/
// fin des fonctions de chargement de texture
/*************************************************************************/

/* general OpenGL initialization function */
bool
initGL ()
{

	/* Enable smooth shading */
	glShadeModel (GL_SMOOTH);

	/* Set the background black */
	glClearColor (0.0f, 0.0f, 0.0f, 0.0f);

	/* Depth buffer setup */
	glClearDepth (1.0f);

	/* Disables Depth Testing */
	glDisable (GL_DEPTH_TEST);
  
  /* Disables lighting*/
  glDisable (GL_LIGHTING);
  
	/* The Type Of Depth Test To Do */
	glDepthFunc (GL_LEQUAL);

	/* Really Nice Perspective Calculations */
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	return (true);
}

extern void dessinerRepere(unsigned int);
/*
void dessinerRepere(unsigned int echelle = 1)
{
    glPushMatrix();
    glScalef(echelle, echelle, echelle);
    glBegin(GL_LINES);

    glColor3ub(255, 0, 0);      //X en rouge
    glVertex3d(0, 0, 0);
    glVertex3d(1, 0, 0);

    glColor3ub(0, 255, 0);      //Y en vert
    glVertex3d(0, 0, 0);
    glVertex3d(0, 1, 0);

    glColor3ub(0, 0, 255);      //Z en bleu
    glVertex3d(0, 0, 0);
    glVertex3d(0, 0, 1);

    glEnd();
    glPopMatrix();
}
*/
/*Class Menu*/
/*private*/

void
Menu::reinit (string ititle, string * listChoices, int nbChoices, int id)
{
	currentChoice = 0;
	delete[]labels;
	initMenu (ititle, listChoices, nbChoices, id);
}

void
Menu::initMenu (string ititle, string * listChoices, int nbChoices, int id)
{
	idMenu = id;
	title = ititle;
	labels = listChoices;
	nbLabels = nbChoices;
}

/*public*/
Menu::Menu (float XHG, float YHG, float XBD, float YBD):
posXHGMenu (XHG),
posYHGMenu (YHG),
posXBDMenu (XBD),
posYBDMenu (YBD)
{
	bool continuer = true;
	SDL_Event event;
  
  idMenu =1 ;
  currentChoice = 0 ;
	//Init GL
	initGL ();

  //chargement du titre
  imageTitre = loadTexture("glouik.jpg",1);
  
	reinitMainMenu ();
	our_font.init ("AXAXAX.TTF", hauteurlettres);
  
	initDisplay ();
  display ();
	//Boucle menu
	while (continuer)
	{
		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
			case SDL_QUIT:
				continuer = false;
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym)
				{
				case SDLK_UP:	//
					precChoice ();
					break;
				case SDLK_DOWN:
					nextChoice ();
					break;
				case SDLK_RETURN:
					activateChoice ();
					break;
				default:
					break;
				}
				break;
			}
      display ();
		}
	}
}

Menu::~Menu ()
{
	delete[]labels;
  our_font.clean();
}

void
Menu::reinitMainMenu ()
{
	string *listlabels;
	listlabels = new string[4];
	listlabels[0] = "Nouvelle Partie";
	listlabels[1] = "Options";
	listlabels[2] = "Credits";
	listlabels[3] = "Quitter";
	initMenu ("Menu Principal", listlabels, 4, 1);
}

void
Menu::reinitPartieType ()
{
	string *listlabels;
	listlabels = new string[3];
	listlabels[0] = "1 joueur";
	listlabels[1] = "2 joueurs cooperatif";
  listlabels[2] = "retour";
	reinit ("Type de Partie", listlabels, 3, 2);
}

void
Menu::reinitOptionsMenu ()
{
	string *listlabels;
	listlabels = new string[4];
	listlabels[0] = "Controles";
	listlabels[1] = "Plein Ecran";
	listlabels[2] = "Son";
  listlabels[3] = "retour";
	reinit ("Options", listlabels, 4, 3);
}

void
Menu::reinitControlesMenu ()
{
	string *listlabels;
	listlabels = new string[7];
	listlabels[0] = "J1: gauche";
	listlabels[1] = "J1: droite";
	listlabels[2] = "J1: action";
	listlabels[3] = "J2: gauche";
	listlabels[4] = "J2: droite";
	listlabels[5] = "J2: action";
  listlabels[6] = "retour";
	reinit ("Controles", listlabels, 7, 4);
}

void
Menu::reinitFullscreen ()
{
	string *listlabels;
	listlabels = new string[2];
	listlabels[0] = "activer/desactiver Plein Ecran";
	listlabels[1] = "retour";
	reinit ("Plein Ecran", listlabels, 2, 5);
}

void
Menu::reinitSoundMenu ()
{
	string *listlabels;
	listlabels = new string[1];
	listlabels[0] = "Volume";
	reinit ("Son", listlabels, 1, 6);
}

void
Menu::nextChoice ()
{
  if(currentChoice < nbLabels-1)
	  currentChoice = (currentChoice + 1) % nbLabels;
}

void
Menu::precChoice ()
{
  if(currentChoice > 0)
	  currentChoice = (currentChoice - 1) % nbLabels;
}

void
Menu::display ()
{
	int i;
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  //dessin du titre
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, imageTitre);
  glColor3ub (0xff, 0xff, 0xff);
  glBegin(GL_QUADS); 
   glTexCoord2d(0,1); glVertex2d(-0.7, 0.9);
   glTexCoord2d(1,1); glVertex2d(0.7, 0.9);
   glTexCoord2d(1,0); glVertex2d(0.7, -0.1);
   glTexCoord2d(0,0); glVertex2d(-0.7, -0.1);
  glEnd();
  glDisable(GL_TEXTURE_2D);

  //Afficher le titre du menu
  glColor3ub (0xFF,0x07,0x07);
  
  freetype::print (our_font, 300, 270, title.c_str ());
	//Afficher le contenu du menu en mettant en surbrillance l'item selectionné
	glPushMatrix ();
  //glTranslatef(80,0,0);
	for (i = 0; i < nbLabels; i++)
	{
		glColor3ub (0, 0, 0xff);
		//glRasterPos2f (posXHGMenu + margin, posYHGMenu + margin);
		//Changer la couleur si c'est le champ selectionné
		if (i == currentChoice)
		{
			glColor3ub (0xff, 0xff, 0xff);
		}
		//Imprimer le string du tableau
    glTranslatef(0,-(hauteurlettres+margin),0);
		freetype::print (our_font, 310, 240, labels[i].c_str ());
	}
	glPopMatrix ();
	SDL_GL_SwapBuffers ();
}

void
Menu::initDisplay ()
{
	//Initialiser les matrices de OpenGL
	/* function to reset our viewport after a window resize */
	/* Height / width ration */
	GLfloat ratio;

	/* Protect against a divide by zero */
	if (HAUTEUR_ECRAN == 0)
		exit(1);
  ratio = ( GLfloat )LARGEUR_ECRAN / ( GLfloat )HAUTEUR_ECRAN;

	/* Setup our viewport. */
	glViewport (0, 0, (GLsizei) LARGEUR_ECRAN, (GLsizei) HAUTEUR_ECRAN);

	/* change to the projection matrix and set our viewing volume. */
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	/* Set our perspective */
  gluOrtho2D(0, 0, LARGEUR_ECRAN, HAUTEUR_ECRAN);
	//gluPerspective (45.0f, ratio, 0.1f, 100.0f);
  //gluLookAt(0,0,20,0,0,0,0,1,0);

	/* Make sure we're chaning the model view and not the projection */
	glMatrixMode (GL_MODELVIEW);
  //gluOrtho2D(0, 0, LARGEUR_ECRAN, HAUTEUR_ECRAN);
	/* Reset The View */
	glLoadIdentity ();
  //gluLookAt(0,0,20,0,0,0,0,1,0);
}

void
Menu::activateChoice ()
{
  Partie *pptr;
	switch (idMenu)
	{
	case 1:		//Menu principal
		switch (currentChoice)
		{
		case 0:	//Nouvelle Partie
			reinitPartieType ();
			break;
		case 1:	//Options
			reinitOptionsMenu ();
			break;
		case 2:	//Credits
			//Montrer les credits A FAIRE
			break;
		case 3:	//Quitter
			exit (EXIT_SUCCESS);
			break;
		default:
			//Erreur!
			cout << "Incorrect position: " << currentChoice <<
				endl;
			exit (EXIT_FAILURE);
		}
		break;
	case 2:		//Menu Type Partie
		switch (currentChoice)
		{
		case 0:	//Charger Partie Normal1j
			//Lancer partie normal1j
      pptr = new Normal1j;
      pptr->start();
      delete pptr;
      initGL();
      initDisplay();
    	SDL_EnableKeyRepeat (1000, 50);
      reinitMainMenu();
			break;
		case 1:	//Charger Partie Coop2j
			//Lancer partie 2j
    /*
      pptr = new Coop2j;
      pptr->start();
      delete pptr;
      initGL();
      initDisplay();
    	SDL_EnableKeyRepeat (1000, 250);
      reinitMainMenu();
    */
			break;
    case 2: //retour vers le menu principal
      reinitMainMenu();
      break;
		default:
			//Erreur!
			cout << "Incorrect position: " << currentChoice <<
				endl;
			exit (EXIT_FAILURE);
		}
		break;
	case 3:		//Menu Options        A TERMINER EVENTUELLEMENT
    switch (currentChoice)
    {
      case 0://Lancer menu controles
        reinitControlesMenu();
        break;
      case 1://Lancer menu Plein Ecran
        reinitFullscreen();
        break;
      case 2://Lancer le menu Son
        reinitSoundMenu();
        break;
      case 3://Retour vers menu principal
        reinitMainMenu();
        break;
      default:
			//Erreur!
			cout << "Incorrect position: " << currentChoice <<
				endl;
			exit (EXIT_FAILURE);
    }
		break;
	case 4:		//Menu Controles
		break;
	case 5:		//Menu Plein Ecran
		break;
	case 6:		//Menu Son
		break;
	}
}
