/***************************************************************************
 *            mixer.h
 *
 *  Tue Dec 06 17:23:55 2007
 *  Copyright  2007  Daniel CALECA
 *  daniel.caleca@etumel.univmed.fr
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <SDL/SDL_mixer.h>
#include <SDL/SDL.h>

class AudioMixer
{
  private:
	bool initialized;
	//The music that will be played
	//Mix_Music *music = NULL;

	//The sound effects that will be used
	Mix_Chunk *high;
	Mix_Chunk *med;
	Mix_Chunk *low;
  	Mix_Chunk *shoot;
    Mix_Chunk *ricochet; 
  	Mix_Chunk *death;  	
  	Mix_Chunk *verre[3];
  
  public:
	AudioMixer()
  	{
		high = NULL;
		med = NULL;
		low = NULL;
  		shoot = NULL;
    	ricochet = NULL; 
  		death = NULL;
		verre[0]=NULL;
		verre[1]=NULL;
		verre[2]=NULL;
		initialized = false;
	}
	~AudioMixer()
  	{    
		//Free the sound effects
    	Mix_FreeChunk( high );
    	Mix_FreeChunk( med );
    	Mix_FreeChunk( low );
		Mix_FreeChunk( shoot );
    	Mix_FreeChunk( ricochet );
    	Mix_FreeChunk( death );
    	Mix_FreeChunk( verre[0] );
    	Mix_FreeChunk( verre[1] );
    	Mix_FreeChunk( verre[2] );
    	//Free the sound effects
    	//Mix_FreeMusic( music );
		
		//Quit SDL_mixer
    	Mix_CloseAudio();
  	}
  
	bool initMixer()
    {
	    //Initialize SDL_mixer
    	if( Mix_OpenAudio( 88200, AUDIO_S16, 2, 4096 ) == -1 )
    	{
        	initialized = false;    
    	}
	      //Load the music
    	//music = Mix_LoadMUS( "beat.wav" );
    
    	//If there was a problem loading the music
    	//if( music == NULL )
    	//{
        //	initialized = false;    
    	//}
	      //Load the sound effects
    	high = Mix_LoadWAV( "sounds/high.wav" );
    	med = Mix_LoadWAV( "sounds/medium.wav" );
    	low = Mix_LoadWAV( "sounds/low.wav" );
	    shoot = Mix_LoadWAV( "sounds/gunshot2.wav" );
     	ricochet = Mix_LoadWAV( "sounds/Ricochet4.wav" ); 
  	 	death = Mix_LoadWAV( "sounds/die.wav" );
	  	verre[0] = Mix_LoadWAV("sounds/verre1.ogg");
	  	verre[1] = Mix_LoadWAV("sounds/verre2.ogg");
	  	verre[2] = Mix_LoadWAV("sounds/verre3.ogg");
	  
	      //If there was a problem loading the sound effects
    	if( ( shoot == NULL ) || ( high == NULL ) || ( med == NULL ) || ( low == NULL )
		   	|| ( ricochet == NULL ) || ( death == NULL ) )
    	{
        	initialized = false;    
    	}
    
	    initialized = true;
		return initialized;
	}
  
	inline void SPlay_BrickHit()
  	{Mix_PlayChannel( -1, med, 0 );}
  
	inline void SPlay_BrickDestroy()
	{
	  int tmp = rand() %3;
	  Mix_PlayChannel( -1, verre[tmp], 0 );
	}
  	 	  	
  
  	inline void SPlay_PlayerDestroy()
	  	{Mix_PlayChannel( -1, death, 0 );}
  
    inline void SPlay_PlayerHit()
	  	{Mix_PlayChannel( -1, low, 0 );}
  
  	inline void SPlay_PlayerFire()
	  	{Mix_PlayChannel( -1, shoot, 0 );}
  
  	inline void SPlay_LesserHit()
	  	{Mix_PlayChannel(-1, low, 0 );}
};
