
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
//using namespace std;
#include "niveau.h"
#include <iostream>
#include <stdio.h>
#include <stdlib.h>

Niveau::Niveau (float briquesY)
{
   int i,j;
  
  qBriques = 0;
  posBriquesY = briquesY;
  numNiveau = 0;
  posBriquesX=-34.5;
  
   for (i = 0; i < nbBriquesX; i++)
   {
      for (j = 0; j < nbBriquesY; j++)
      {
         tabBriques[i][j] = NULL;
      }  
   }
}

Niveau::~Niveau()
{
   int i,j;
   for (i = 0; i < nbBriquesX; i++)
   {
      for (j = 0; j < nbBriquesY; j++)
      {
         if(tabBriques[i][j] != NULL)
            delete tabBriques[i][j];
      }  
   }
}

void Niveau::detruire (int i, int j)
{
   delete tabBriques[i][j];
   tabBriques[i][j] = NULL;
   qBriques = qBriques - 1;
}

void Niveau::defaultLevel()
{
  int i,j;
  for (i = 0; i < nbBriquesY; i++)
  {
    for (j = 0; j < nbBriquesX; j++)
    {
      //nettoyage de l'ancien niveau
      if(tabBriques[j][i] != NULL)
        delete tabBriques[j][i];
      
      tabBriques[j][i] = new Brique (1+j,
                                     j * ecartX + posBriquesX,
                                     i * ecartY + posBriquesY);
      
        qBriques++;
    }
  }
}

void Niveau::initBriques (const char* fichier, int nNiveau)
{
  FILE *scriptFile;
  int numNiveau,
  i,
  j,
  temoinLecture,
  valeurCase;
  
  qBriques = 0;
  //ouvrir le fichier
  scriptFile = fopen (fichier, "r");
  if(!scriptFile)
  {
    std::cerr << "le fichier " << fichier << " est introuvable" << std::endl;
    std::cerr << "on bascule sur un niveau par defaut" << std::endl;
    defaultLevel();
    return;
  }
  
  //chercher le [numNiveau]
  //si existe
  temoinLecture = fscanf (scriptFile, "[%d]", &numNiveau);
  if (temoinLecture == EOF)
  {
    //Erreur Lecture
    //throw FileException("Unable to read level file");
    std::cout << "Level file unreadable" << std::endl;
  }
  if (numNiveau != nNiveau)
  {
    //throw FileException("script file level number mismatch");
    std::cout << "Level mismatch numNiveau=" << numNiveau << "; nNiveau=" << nNiveau << std::endl;
    exit (EXIT_FAILURE);
  }
  //lire la matrice, remplir le tableau des briques, incrementer qBriques
  for (i = 0; i < nbBriquesY; i++)
  {
    for (j = 0; j < nbBriquesX; j++)
    {
      temoinLecture = fscanf (scriptFile, "%d", &valeurCase);
      if (temoinLecture == EOF)
      {
        //Erreur Lecture
        //throw FileException("Unexpected termination of level script file");
        std::cout << "Abnormal termination for the level " << numNiveau << " file" << std::endl;
      }
      
      //nettoyage de l'ancien niveau
      if(tabBriques[j][i] != NULL)
        delete tabBriques[j][i];
      
      //Instancier la brique selon la valeurCase
      if (valeurCase == 0)
      {
        tabBriques[j][i] = NULL;
      }
      else
      {
        tabBriques[j][i] = new Brique (valeurCase,
                                       j * ecartX + posBriquesX,
                                       i * ecartY + posBriquesY);
        
        if (valeurCase != -1)	//On ne compte pas les briques incassables
          qBriques++;
      }
    }
  }
  //fermer le fichier
  if(scriptFile != NULL)
    fclose (scriptFile);
}
