
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <list>
#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "partie.h"
#include <iostream>

Partie::Partie (int nbJ, float hBriques, Side coteJ1,Side coteJ2):
niveauCourant (hBriques),
disposition (this, 0, 0, 650, 600),
joueur1(1),
joueur2(2)
{
  nbJoueur = nbJ;
  joueur1.setSide (coteJ1);
  joueur2.setSide (coteJ2);
  
  newBall(&joueur1, true);
  if(nbJoueur == 2)
    newBall(&joueur2, true);
  
  disposition.setJoueur(&joueur1 , 1);
  disposition.setJoueur(&joueur2 , 2);
}

void Partie::start()
{
    int start_time;
    bool continuer = true;
    SDL_Event event;
  
    SDL_EnableKeyRepeat(1, 1);

    initAffichage();
    //boucle principale
    while (continuer)
    {
        start_time = SDL_GetTicks();
        while (SDL_PollEvent(&event))
        {
            switch (event.type)
            {
            case SDL_QUIT:
                continuer = false;
                break;
            case SDL_KEYDOWN:
                switch (event.key.keysym.sym)
                {
                case SDLK_LEFT:
                    deplacerJ1(-1);
                    break;
                case SDLK_RIGHT:
                    deplacerJ1(1);
                    break;
                case SDLK_a:
                    deplacerJ2(-1);
                    break;
                case SDLK_z:
                    deplacerJ2(1);
                    break;

                case SDLK_SPACE:
                    lancerBallesJ1();
                    break;
                case SDLK_q:
					continuer = false;
                	break;
                default:
                    break;
                }
                break;
            }
        }
         
        //deplace et gere les collisions
        gameHandler();
         
        //nouvelle frame
        redessiner();

        while (SDL_GetTicks() - start_time < 1000 / 60)
        {
            SDL_Delay(1);
        }

    }
}

void Partie::deplacements ()
{
  //deplacer Debris
  std::list < Debris >::iterator i;
  std::list < Debris >::iterator irm;
  for (i = listeDebris.begin (); i != listeDebris.end (); i++)
  {
    if(!(i->isTooOld()))
      i->deplacer ();
    else
      while(i != listeDebris.end())
      {
        irm = i;
        i++;
        listeDebris.erase(irm);
      }
  }
  //deplacer Balles
  std::list < Balle >::iterator j;
  for (j = listeBalles.begin (); j != listeBalles.end (); j++)
  {
    if(!(j->isHooked()))
      j->deplacer ();
  }

  //deplacer Bonus
  std::list < Bonus >::iterator k;
  for (k = listeBonus.begin (); k != listeBonus.end (); k++)
  {
    k->deplacer ();
  }
}

void Partie::gameHandler ()
{
  int numNiveau;

  if (niveauCourant.nbBriquesRestantes() == 0)
  {
    numNiveau = niveauCourant.getNumNiveau ();
    niveauCourant.initBriques ("niveaux.lvl", numNiveau + 1);
  }
  else
  {
    deplacements ();
    verifPositions ();
  }
}

void Partie::newBall(Joueur *owner, bool accrochee)
{
  Batte batte = owner->getBatteRef();    
  //probleme si la batte est en haut
  listeBalles.push_back (*new Balle(owner,batte.getPosition()->X,
                                    batte.getPosition()->Y + batte.getBoiteCol()->lY,
                                    accrochee));
  owner->addBalle();
}

//afficher tous les objets
void Partie::redessiner ()
{
  //clean + dessine le HUD + entre dans le viewport du jeu
  disposition.initFrame ();

  //dessine les balles
  std::list < Balle >::iterator iBalles;
  for (iBalles = listeBalles.begin (); iBalles != listeBalles.end (); iBalles++)
    iBalles->afficher ();
  
  //dessine les briques
  int i,j;
  for (i = 0; i < niveauCourant.nbBriquesX; i++)
  {
    for (j = 0; j < niveauCourant.nbBriquesY; j++)
    {
      if (niveauCourant.tabBriques[i][j] != NULL)
        niveauCourant.tabBriques[i][j]->afficher ();
    }
  }
  
  glDisable(GL_CULL_FACE);
  //dessine les debris
  std::list < Debris >::iterator iDebris;
  for (iDebris = listeDebris.begin (); iDebris != listeDebris.end (); iDebris++)
    iDebris->afficher ();
  glEnable(GL_CULL_FACE);
  
  //dessine les battes
  if (nbJoueur == 1)
  {
    if(!joueur1.isDestroyed())
      joueur1.afficher ();
  }
  else if (nbJoueur == 2)
  {
    if(!joueur1.isDestroyed())
      joueur1.afficher ();
    if(!joueur2.isDestroyed())
      joueur2.afficher ();
  }

  //dessine les bonus
  std::list < Bonus >::iterator iBonus;
  for (iBonus = listeBonus.begin (); iBonus != listeBonus.end (); iBonus++)
    iBonus->afficher ();
  
  glFlush();
  SDL_GL_SwapBuffers();
}

/*** destructions d'objets particuliers ***/
void Partie::detruireBrique (int i, int j)
{
  niveauCourant.detruire (i,j);
}

void Partie::detruire (Debris * cible){}
void Partie::detruire (Bonus * cible){}
void Partie::detruire (Balle * cible){}
/******************************************/

/***** verification des collisions entre objets *****/
bool Partie::eventCollision (Balle & balle, Batte & batte)
{
  float rayon = balle.getRayon ();
  Vecteur *posBalle = balle.getPosition ();
  Vecteur *posBatte = batte.getPosition ();

  if (posBalle->X + rayon < posBatte->X - batte.getBoiteCol ()->lX / 2.0)
  {
    //balle trop a gauche
    return false;
  }

  if (posBalle->X - rayon > posBatte->X + batte.getBoiteCol ()->lX / 2.0)
  {
    //balle trop a droite
    return false;
  }

  if (posBalle->Y - rayon > posBatte->Y + batte.getBoiteCol ()->lY / 2.0)
  {
    //balle trop haute
    return false;
  }

  if (posBalle->Y + rayon < posBatte->Y - batte.getBoiteCol ()->lY / 2.0)
  {
    //balle trop basse
    return false;
  }

  return true;
}

bool Partie::eventCollision (Balle & balle, Balle & balle2)
{
  float rayon = balle.getRayon () + balle2.getRayon ();
  Vecteur *posBalle = balle.getPosition ();
  Vecteur *posBalle2 = balle2.getPosition ();

  float X,
    Y;

  X = posBalle->X - posBalle2->X;
  X = (X < 0) ? -X : X;

  Y = posBalle->Y - posBalle2->Y;
  Y = (Y < 0) ? -Y : Y;

  if (X * X + Y * Y < rayon * rayon)
    return true;
  else
    return false;
}

bool Partie::eventCollision (Brique & brique, Balle & balle)
{
  float rayon = balle.getRayon ();
  Vecteur *posBalle = balle.getPosition ();
  Vecteur *posBrique = brique.getPosition ();

  if ((posBalle->X + rayon) < (posBrique->X - (brique.getBoiteCol ()->lX / 2.0)))
  {
    //balle trop a gauche
    return false;
  }

  if ((posBalle->X - rayon) > (posBrique->X + (brique.getBoiteCol ()->lX / 2.0)))
  {
    //balle trop a droite
    return false;
  }

  if ((posBalle->Y - rayon) > (posBrique->Y + (brique.getBoiteCol ()->lY / 2.0)))
  {
    //balle trop haute
    return false;
  }

  if ((posBalle->Y + rayon) < (posBrique->Y - (brique.getBoiteCol ()->lY / 2.0)))
  {
    //balle trop basse
    return false;
  }
  return true;
}

bool Partie::eventCollision (Bonus & bonus, Batte & batte)
{
  float bonusH = bonus.getBoiteCol ()->lY / 2.0;
  float bonusW = bonus.getBoiteCol ()->lX / 2.0;

  float batteH = batte.getBoiteCol ()->lY / 2.0;
  float batteW = batte.getBoiteCol ()->lX / 2.0;

  Vecteur *posBonus = bonus.getPosition ();
  Vecteur *posBatte = batte.getPosition ();

  if (posBonus->X + bonusW < posBatte->X - batteW)
  {
    //bonus trop a gauche
    return false;
  }

  if (posBonus->X - bonusW > posBatte->X + batteW)
  {
    //bonus trop a droite
    return false;
  }

  if (posBonus->Y - bonusH > posBatte->Y + batteH)
  {
    //bonus trop haut
    return false;
  }

  if (posBonus->Y + bonusH < posBatte->Y - batteH)
  {
    //bonus trop bas
    return false;
  }

  return true;

}

/************************************************/
