/***************************************************************************
 *            partie.h
 *
 *  Thu Nov 24 11:05:07 2007
 *  Copyright  2007  Daniel CALECA
 *  daniel.caleca@etumel.univmed.fr
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef H_CB_PARTIE
#define H_CB_PARTIE

#include <list>
#include <string>

#include "joueur.h"
#include "display.h"

#include "bonus.h"
#include "balle.h"
#include "brique.h"
#include "batte.h"
#include "debris.h"

#include "niveau.h"

class Partie
{
  protected:
	Joueur joueur1;
  Joueur joueur2;
  int nBalles;
  Niveau niveauCourant;
  Disposition disposition;
  std::list < Balle > listeBalles;
  std::list < Bonus > listeBonus;
  std::list < Debris > listeDebris;
  
  int nbJoueur;
  void detruireBrique(int i, int j);
  void detruire (Debris * cible);
  void detruire (Bonus * cible);
  void detruire (Balle * cible);
  
  void deplacements ();
  void gameHandler ();
  void redessiner ();
  
  bool eventCollision (Balle & balle, Batte & batte);
  bool eventCollision (Balle & balle, Balle & balle2);
  bool eventCollision (Brique & brique, Balle & balle);
  bool eventCollision (Bonus & bonus, Batte & batte);
  
  virtual void verifPositions () = 0;

    public:
  Partie (int nbJoueurs, float hBriques, Side coteJ1, Side coteJ2);
  virtual ~Partie(){}

  void start();

  void newBall(Joueur *owner,bool accrochee);
  virtual void newBonus(float X, float Y, int sens) =0;
  
  inline void deplacerJ1 (int sens)
  {
    if(((joueur1.getBatteRef().getPosition()->X + joueur1.getBatteRef().getBoiteCol()->lX /2. 
            <= 40 - joueur1.getBatteRef().getPasDep()) && (sens > 0))
       || ((joueur1.getBatteRef().getPosition()->X - joueur1.getBatteRef().getBoiteCol()->lX /2. 
            >= -40 + joueur1.getBatteRef().getPasDep()) && (sens < 0)))
    {
      joueur1.deplacer (sens);
      
      //deplacer les balles accrochees
      std::list < Balle >::iterator j;
      for (j = listeBalles.begin (); j != listeBalles.end (); j++)
      {
        if(j->getProp() == &joueur1 && j->isHooked())
        {
          j->suivreBatte(sens);
        }
      }
    } 
  }
  
  inline void deplacerJ2 (int sens)
  {
    if(((joueur2.getBatteRef().getPosition()->X + joueur2.getBatteRef().getBoiteCol()->lX /2. 
            <= 40 - joueur2.getBatteRef().getPasDep()) && (sens > 0))
       || ((joueur2.getBatteRef().getPosition()->X - joueur2.getBatteRef().getBoiteCol()->lX /2. 
            >= -40 + joueur2.getBatteRef().getPasDep()) && (sens > 0)))
    {
      joueur2.deplacer (sens);
      
      //deplacer les balles accrochees
      std::list < Balle >::iterator j;
      for (j = listeBalles.begin (); j != listeBalles.end (); j++)
      {
        if(j->getProp() == &joueur2 && j->isHooked())
        {
          j->suivreBatte(sens);
        }
      }
    } 
  }

  inline void initAffichage ()
  {
      disposition.initAffichage ();
  }
  
  inline void lancerBallesJ1()
  {
    std::list < Balle >::iterator j;
    for (j = listeBalles.begin (); j != listeBalles.end (); j++)
    {
      if(j->getProp() == &joueur1 && j->isHooked())
      {
        j->lancer();
      }
    }
  }
};

#endif
